from __future__ import unicode_literals

import re

from django import forms

from .models import Youtube
from .widgets import YoutubeVideoURLWidget


class YoutubeModelForm(forms.ModelForm):

    class Meta:
        model = Youtube
        fields = '__all__'
        widgets = {
            'video_url': YoutubeVideoURLWidget(),
        }

    def clean_video_url(self):
        video_url = self.cleaned_data['video_url']
        pattern = (
            r'(https?://)?(www\.)?'
            '(youtube|youtu|youtube-nocookie)\.(com|be)/'
            '(watch\?v=|embed/|v/|.+\?v=)?([\w-]{11})')

        match = re.match(pattern, video_url)
        if not match:
            raise forms.ValidationError('The YouTube video URL is invalid')

        return video_url



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.http import HttpResponseNotAllowed
from django.template.loader import select_template
from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .conf import settings
from .exceptions import YoutubeAPIError
from .forms import YoutubeModelForm
from .models import Youtube
from .utils import get_video_details

try:
    from django.http import JsonResponse
except ImportError:
    from .compat import JsonResponse


class YoutubePlugin(CMSPluginBase):
    model = Youtube
    form = YoutubeModelForm

    module = settings.DJANGOCMS_YOUTUBE_PLUGIN_MODULE
    name = settings.DJANGOCMS_YOUTUBE_PLUGIN_NAME
    render_template = settings.DJANGOCMS_YOUTUBE_TEMPLATES[0][0]

    text_enabled = settings.DJANGOCMS_YOUTUBE_TEXT_ENABLED
    page_only = settings.DJANGOCMS_YOUTUBE_PAGE_ONLY
    require_parent = settings.DJANGOCMS_YOUTUBE_REQUIRE_PARENT
    parent_classes = settings.DJANGOCMS_YOUTUBE_PARENT_CLASSES
    allow_children = settings.DJANGOCMS_YOUTUBE_ALLOW_CHILDREN
    child_classes = settings.DJANGOCMS_YOUTUBE_CHILD_CLASSES

    def get_fieldsets(self, request, obj=None):
        if settings.DJANGOCMS_YOUTUBE_FIELDSETS:
            return settings.DJANGOCMS_YOUTUBE_FIELDSETS

        advanced_option_fields = ('theme', )
        if settings.DJANGOCMS_YOUTUBE_ENABLE_CUSTOM_VIDEO_SIZE:
            advanced_option_fields += ('width', 'height', )

        if len(settings.DJANGOCMS_YOUTUBE_TEMPLATES) > 1:
            advanced_option_fields += ('plugin_template', )

        fieldsets = (
            (None, {
                'fields': ('video_url', )
            }),
            (None, {
                'fields': ('thumbnail',)
            }),
            (None, {
                'fields': ('title', 'description', 'description_option', )
            }),
            (None, {
                'fields': advanced_option_fields
            }),
            (_('Video Meta'), {
                'classes': ('advanced', 'collapse'),
                'fields': ('video_data', ),
            }),
        )

        return fieldsets

    def get_render_template(self, context, instance, placeholder):
        # returns the first template that exists, falling back to bundled template
        return select_template([
            instance.plugin_template,
            settings.DJANGOCMS_YOUTUBE_TEMPLATES[0][0],
            'djangocms_youtube/default.html'
        ])

    def get_model_info(self):
        # module_name is renamed to model_name in Django 1.8
        app_label = self.model._meta.app_label
        try:
            return app_label, self.model._meta.model_name
        except AttributeError:
            return app_label, self.model._meta.module_name

    def get_plugin_urls(self):
        from django.conf.urls import patterns, url
        info = self.get_model_info()

        return patterns(
            '',
            url(r'^gdata/$',
                admin.site.admin_view(self.youtube_data_api),
                name='%s_%s_gdata' % info),
        )

    def youtube_data_api(self, request):
        if not request.method == 'POST':
            return HttpResponseNotAllowed(['POST', ])

        video_id = request.POST.get('video_id', None)
        if video_id is None:
            response = {
                'status': 400,
                'error_type': 'idRequired',
                'error_message': '',
            }
            return JsonResponse(response, status=400)

        try:
            response = get_video_details(video_id)
        except YoutubeAPIError as e:
            response = {
                'status': e.status_code,
                'error_type': e.error_type,
                'error_message': e.error_message,
            }
            return JsonResponse(response, status=e.status_code)
        else:
            return JsonResponse(response)

    class Media:
        css = {
            'all': (
                'admin/css/djangocms_youtube/changeform.css',
            )
        }
        js = ('admin/js/djangocms_youtube/changeform.js', )

plugin_pool.register_plugin(YoutubePlugin)



import json

from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse


class JsonResponse(HttpResponse):

    def __init__(self, data, encoder=DjangoJSONEncoder, safe=True, **kwargs):
        if safe and not isinstance(data, dict):
            raise TypeError('In order to allow non-dict objects to be '
                            'serialized set the safe parameter to False')
        kwargs.setdefault('content_type', 'application/json')
        data = json.dumps(data, cls=encoder)
        super(JsonResponse, self).__init__(content=data, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion

from filer.fields.image import FilerImageField
from jsonfield import JSONField

from djangocms_youtube.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '__latest__'),
        ('cms', '__latest__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Youtube',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('title', models.CharField(max_length=150, verbose_name='Title', blank=True)),
                ('video_url', models.URLField(help_text='Paste the URL of the YouTube video', verbose_name='Video URL')),
                ('width', models.PositiveIntegerField(help_text='Sets the width of your player, used on some templates where applicable', null=True, verbose_name='Width', blank=True)),
                ('height', models.PositiveIntegerField(help_text='Sets the height of your player, used on some templates where applicable', null=True, verbose_name='Height', blank=True)),
                ('description', models.TextField(help_text='You can add a Description to your video, to be displayed beneath your video on your page.', null=True, verbose_name='Video Description', blank=True)),
                ('description_option', models.CharField(default=settings.DJANGOCMS_YOUTUBE_DESCRIPTION_CHOICES[0][0], max_length=50, verbose_name='Description Option', blank=True, choices=settings.DJANGOCMS_YOUTUBE_DESCRIPTION_CHOICES)),
                ('theme', models.CharField(default=settings.DJANGOCMS_YOUTUBE_DEFAULT_THEME, max_length=100, verbose_name='Theme', choices=settings.DJANGOCMS_YOUTUBE_THEME_CHOICES)),
                ('plugin_template', models.CharField(default=settings.DJANGOCMS_YOUTUBE_TEMPLATES[0][0], max_length=255, verbose_name='Template', choices=settings.DJANGOCMS_YOUTUBE_TEMPLATES)),
                ('video_data', JSONField(help_text='For advanced users only \u2014 please do not edit this data unless you know what you are doing.', null=True, verbose_name='YouTube Data', blank=True)),
                ('thumbnail', FilerImageField(related_name='djangocms_youtube_thumbnails', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='filer.Image', help_text='Image Overlay - this image will display over the video on your site and allow users to see an image of your choice before playing the video.', null=True, verbose_name='Custom Thumbnail')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from django.contrib.sites.models import Site
from django.db import models
from django.utils import six
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from cms.models import CMSPlugin
from filer.fields.image import FilerImageField
from isodate import parse_datetime, parse_duration
from jsonfield import JSONField

from .conf import settings

logger = logging.getLogger('djangocms_youtube')


@python_2_unicode_compatible
class Youtube(CMSPlugin):
    title = models.CharField(_('Title'), max_length=150, blank=True)
    thumbnail = FilerImageField(
        verbose_name=_('Custom Thumbnail'), blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name='djangocms_youtube_thumbnails',
        help_text=_('Image Overlay - this image will display over the '
                    'video on your site and allow users to see an image '
                    'of your choice before playing the video.'))
    video_url = models.URLField(
        _('Video URL'), help_text=_('Paste the URL of the YouTube video'))

    width = models.PositiveIntegerField(
        _('Width'), blank=True, null=True,
        help_text=_('Sets the width of your player, '
                    'used on some templates where applicable'))
    height = models.PositiveIntegerField(
        _('Height'), blank=True, null=True,
        help_text=_('Sets the height of your player, '
                    'used on some templates where applicable'))

    description = models.TextField(
        _('Video Description'), blank=True, null=True,
        help_text=_('You can add a Description to your video, to be '
                    'displayed beneath your video on your page.'))

    description_option = models.CharField(
        _('Description Option'), max_length=50,
        choices=settings.DJANGOCMS_YOUTUBE_DESCRIPTION_CHOICES,
        default=settings.DJANGOCMS_YOUTUBE_DESCRIPTION_CHOICES[0][0], blank=True)

    theme = models.CharField(
        _('Theme'), max_length=100,
        choices=settings.DJANGOCMS_YOUTUBE_THEME_CHOICES,
        default=settings.DJANGOCMS_YOUTUBE_DEFAULT_THEME,
    )
    plugin_template = models.CharField(
        _('Template'), max_length=255,
        choices=settings.DJANGOCMS_YOUTUBE_TEMPLATES,
        default=settings.DJANGOCMS_YOUTUBE_TEMPLATES[0][0],
    )

    video_data = JSONField(
        verbose_name=_('YouTube Data'), blank=True, null=True,
        help_text=_('For advanced users only — please do not edit '
                    'this data unless you know what you are doing.')
    )

    def __str__(self):
        return self.title

    @property
    def video(self):
        cls = Video(**self.video_data)
        return cls

    def _generate_thumbnails(self):
        _thumbnails = {}
        for name, opts in six.iteritems(settings.DJANGOCMS_YOUTUBE_THUMBNAIL_SIZES):
            try:
                thumb_opts = {
                    'size': (int(opts['width']), int(opts['height'])),
                    'subject_location': self.thumbnail.subject_location,
                    'crop': False,
                    'upscale': True,
                }
                thumb = self.thumbnail.file.get_thumbnail(thumb_opts)
                _thumbnails[name] = {
                    'url': thumb.url,
                    'width': opts['width'],
                    'height': opts['height']
                }
            except Exception as e:
                logger.error('Error while generating thumbnail: %s', e)
        return _thumbnails

    @property
    def highest_resolution_thumbnail(self):
        thumbnails = self.get_thumbnails()
        for size in ('maxres', 'standard', 'high', 'medium', 'default',):
            if size in thumbnails:
                return thumbnails[size]

    def get_thumbnails(self):
        if self.thumbnail_id is None:
            return self.video.get_thumbnails()

        return self._generate_thumbnails()

    def get_title(self):
        if self.title:
            return self.title
        return self.video.get_title()

    def get_description(self):
        return self.description

    def get_thumbnail(self):
        thumbnail = self.highest_resolution_thumbnail.get('url', '')
        protocol = 'https' if settings.DJANGOCMS_YOUTUBE_USE_HTTPS else 'http'
        if thumbnail.startswith('http'):
            return thumbnail

        current_site = Site.objects.get_current()
        return '{protocol}://{domain}{thumbnail}'.format(
            protocol=protocol,
            domain=current_site.domain,
            thumbnail=thumbnail
        )


@python_2_unicode_compatible
class Video(object):

    def __init__(self, *args, **kwargs):
        for key, value in six.iteritems(kwargs):
            setattr(self, key, value)

    def get_id(self):
        return getattr(self, 'id')

    def get_title(self):
        return self.snippet.get('title')

    def get_description(self):
        return self.snippet.get('description')

    def get_channel_title(self):
        return self.snippet.get('channelTitle')

    def get_embed_html(self):
        return self.player.get('embedHtml')

    def get_embed_url(self):
        return 'https://www.youtube.com/embed/{video_id}'.format(video_id=self.get_id())

    def get_thumbnails(self):
        return self.snippet.get('thumbnails', {})

    def get_duration(self):
        return self.contentDetails.get('duration')

    def get_duration_seconds(self):
        duration = parse_duration(self.get_duration())
        return duration.total_seconds()

    def get_published_at(self):
        return self.snippet.get('publishedAt')

    def get_published_datetime(self):
        return parse_datetime(self.get_published_at())

    def get_tags(self):
        return self.snippet.get('tags', [])

    def __str__(self):
        return 'Video: %s' % self.id



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from requests.api import request
from requests.exceptions import RequestException

from .conf import settings
from .exceptions import YoutubeAPI404, YoutubeAPIError


def fetch_gdata(resource, parts, identifier):
    access_token = settings.DJANGOCMS_YOUTUBE_API_KEY

    if access_token is None:
        msg = _('Missing DJANGOCMS_YOUTUBE_API_KEY settings')
        raise YoutubeAPIError(500, 'ImproperlyConfigured', force_text(msg))

    url_template = '{resource}?part={parts}&fields=items&maxResults=1&id={id}&key={access_token}'
    url = url_template.format(
        resource=resource,
        parts=','.join(parts),
        id=identifier,
        access_token=access_token,
    )

    base_url = 'https://www.googleapis.com/youtube/v3/'
    try:
        response = request('get', url='{base_url}{url}'.format(base_url=base_url, url=url))
        response.raise_for_status()
    except RequestException as e:
        error = e.response.json().get('error')
        raise YoutubeAPIError(error['code'], 'YoutubeAPIError', error['message'])
    else:
        items = response.json().get('items', [])
        if not items:
            raise YoutubeAPI404()
        return items[0]


def get_video_details(video_id):
    parts = ('snippet', 'player', 'status', 'contentDetails', )
    try:
        result = fetch_gdata('videos', parts, video_id)
    except YoutubeAPI404:
        msg = _('This video does not exist.')
        raise YoutubeAPIError(404, 'videoNotFound', force_text(msg))
    else:
        category_name = get_category_name(result.get('snippet', {}).get('categoryId', ''))
        result['snippet']['categoryName'] = category_name
        return result


def get_category_name(cat_id):
    parts = ('snippet', )
    try:
        result = fetch_gdata('videoCategories', parts, cat_id)
    except YoutubeAPI404:
        msg = _('This video category does not exist.')
        raise YoutubeAPIError(404, 'videoCategoryNotFound', force_text(msg))
    else:
        return result.get('snippet', {}).get('title', '')



from django.contrib.sitemaps.views import x_robots_tag
from django.contrib.sites.shortcuts import get_current_site
from django.template.response import TemplateResponse

from .video_sitemap import VideoElement


@x_robots_tag
def video_sitemap(request, sitemaps,
                  template_name='djangocms_youtube/sitemap.xml',
                  content_type='application/xml'):

    protocol = request.scheme

    site = get_current_site(request)
    domain = site.domain

    urls = []

    for section, site in sitemaps.items():
        if callable(site):
            site = site()

        for url in site.get_urls():
            location = '{protocol}://{domain}{location}'.format(
                protocol=protocol, domain=domain, location=url.get('location'))
            videos = []
            for plugin in url.get('youtube_plugins', []):
                videos.append(VideoElement(plugin))
            if videos:
                url_info = {
                    'location': location,
                    'videos': videos,
                }
                urls.append(url_info)

    response = TemplateResponse(
        request, template_name, {'urlset': urls}, content_type=content_type)
    return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.core.urlresolvers import reverse

from .models import Youtube


class YoutubeVideoURLWidget(forms.TextInput):
    model = Youtube

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}

        opts = self.model._meta

        app_label = opts.app_label
        try:
            model_name = opts.model_name
        except AttributeError:
            model_name = opts.module_name

        attrs['data-gdata'] = reverse('admin:%s_%s_%s' % (app_label, model_name, 'gdata',))
        return super(YoutubeVideoURLWidget, self).render(name, value, attrs)



# -*- coding: utf-8 -*-

from django.conf import settings  # noqa
from django.utils.translation import ugettext_lazy as _

from appconf import AppConf


class DjangoCMSYoutubeConf(AppConf):
    PLUGIN_MODULE = _('Generic')
    PLUGIN_NAME = _('Youtube')

    FIELDSETS = None
    PAGE_ONLY = False
    PARENT_CLASSES = None
    REQUIRE_PARENT = False
    TEXT_ENABLED = False
    ALLOW_CHILDREN = True
    CHILD_CLASSES = None
    USE_HTTPS = False
    API_KEY = None

    ENABLE_CUSTOM_VIDEO_SIZE = False

    TEMPLATES = (
        ('djangocms_youtube/default.html', _('Default')),
    )

    DESCRIPTION_CHOICES = (
        ('hidden', _('Do Not Display Description')),
        ('below', _('Description Below the Video')),
    )

    DEFAULT_HEIGHT = 480
    DEFAULT_WIDTH = 854

    DEFAULT_THEME = 'light'
    THEME_CHOICES = (
        ('dark', _('Dark')),
        ('light', _('Light')),
    )

    THUMBNAIL_SIZES = {
        'default': {
            'width': 120,
            'height': 90
        },
        'high': {
            'width': 480,
            'height': 360
        },
        'medium': {
            'width': 320,
            'height': 180
        },
        'maxres': {
            'width': 1280,
            'height': 720
        },
        'standard': {
            'width': 640,
            'height': 480
        }
    }

    class Meta:
        prefix = 'djangocms_youtube'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Q
from django.utils import translation

from cms.models import Title
from cms.models.fields import PlaceholderField

from .models import Youtube


class VideoElement(object):
    def __init__(self, plugin):
        self.plugin = plugin

    def get_thumbnail(self):
        return self.plugin.get_thumbnail()

    def get_title(self):
        return self.plugin.get_title()

    def get_description(self):
        return self.plugin.get_description()

    def get_embed_url(self):
        return self.plugin.video.get_embed_url()

    def get_duration(self):
        return int(self.plugin.video.get_duration_seconds())

    def get_publication_date(self):
        return self.plugin.video.get_published_at()

    def get_tags(self):
        return self.plugin.video.get_tags()


class VideoSitemap(object):
    model = None

    def __init__(self, *args, **kwargs):
        self.language = kwargs.pop('language', None)
        super(VideoSitemap, self).__init__(*args, **kwargs)

    def get_language(self):
        return self.language or translation.get_language()

    def _get_youtube_plugins(self, obj):
        youtube_plugins = []
        for field in self.model._meta.fields:
            if isinstance(field, PlaceholderField):
                placeholder = getattr(obj, field.name, None)
                plugins = Youtube.objects.filter(
                    placeholder=placeholder, language=self.get_language())
                youtube_plugins.extend(plugins)
        return youtube_plugins

    def get_queryset(self):
        if self.model is not None:
            queryset = self.model._default_manager.all()
        else:
            raise ImproperlyConfigured(
                '{cls} is missing a QuerySet. '
                'Define {cls}.model or override {cls}.get_queryset()'.format(
                    cls=self.__class__.__name__)
            )
        return queryset

    def location(self, item):
        item.get_absolute_url()

    def get_urls(self):
        urls = []
        for obj in self.get_queryset():
            youtube_plugins = self._get_youtube_plugins(obj)
            if youtube_plugins:
                urls.append({
                    'location': self.location(obj),
                    'youtube_plugins': youtube_plugins,
                })
        return urls


class CMSVideoSitemap(VideoSitemap):
    model = Title

    def get_queryset(self):
        queryset = self.model._default_manager.public().filter(
            Q(redirect='') | Q(redirect__isnull=True),
            page__login_required=False,
            page__site=Site.objects.get_current(),
        ).order_by('page__path')
        return queryset

    def location(self, title):
        translation.activate(title.language)
        url = title.page.get_absolute_url(title.language)
        translation.deactivate()
        return url

    def get_urls(self):
        urls = []
        for title in self.get_queryset():
            youtube_plugins = Youtube.objects.filter(
                placeholder__page=title.page, language=title.language)
            if youtube_plugins.exists():
                urls.append({
                    'location': self.location(title),
                    'youtube_plugins': youtube_plugins,
                })
        return urls





class YoutubeAPIError(Exception):
    def __init__(self, status_code, error_type, error_message, *args, **kwargs):
        self.status_code = status_code
        self.error_type = error_type
        self.error_message = error_message

    def __str__(self):
        return '(%s) %s - %s' % (self.status_code, self.error_type, self.error_message)


class YoutubeAPI404(Exception):
    pass
